(function ($) {

/*** This document contains theme functions specific to menus and the main menu region ***/

  /*****************************************************
  *** main menu behavior
  *****************************************************/

  Drupal.behaviors.wcmOmegaAccessibleMenu = {
    attach: function (context, settings) {

      function navigate(group, index) {
        if (index < 0) {
          index = group.length - 1;
        }
        else if (index >= group.length) {
          index = 0;
        }

        group.get(index).focus();
      }

      function isL1Item(item) {
        return $(item).hasClass('sf-depth-1');
      }

      function isL2Item(item) {
        return $(item).hasClass('sf-depth-2');
      }

      function isParentItem(item) {
        return $(item).hasClass('menuparent');
      }

      var menu = $('ul#superfish-1');
      var items = menu.find('a, .nolink');
      var parents = items.filter('.sf-depth-1');

      items.on('keydown', function(e){
        var isL1 = isL1Item(this);
        var isL2 = isL2Item(this);
        var isParent = isParentItem(this);
        var key = e.which;
        var children;

        if (key > 36 && key < 41) {
          e.preventDefault();
          var index = null;

          // left/right keys
          if (key % 2) {
            if (isL1) {
              index = parents.index(this);
            }
            else if (isL2)  {
              index = parents.index($(this).parents('li.sf-depth-1').find('a.sf-depth-1'));
            }

            index = index + key - 38;
            navigate(parents, index);
          }
          // up/down keys
          else {
            children = $(this).parents('.main-menu-second-ul').find('a');

            if (!children.length) {
              children = $(this).next('.main-menu-second-ul').find('a');
            }
            index = children.index(this) + key - 39;

            if (isParent || !isL1) {
              navigate(children, index);
            }
          }
        }
        // ESC Key
        else if (key == 27) {
          e.preventDefault();

          if (!(isL1 && isParent)) {
            $(this).parents('li.sf-depth-1').find('.sf-depth-1').focus();
          }
        }
        // SPACE key
        else if (key == 32) {
          e.preventDefault();
          this.click();
        }
        // ENTER key
        else if (key == 13) {
          if (isL1 && isParent) {
            e.preventDefault();

            children = $(this).parents('ul').find('a.sf-depth-2');
            navigate(children, 0);
          }
        }
      });

      parents.on('keydown', function(e){
        var key = e.which;
        if (key == 27) {
          e.preventDefault();
          $(this).parent('li').hideSuperfishUl();
        }
      });
    }
  };

  Drupal.behaviors.wcmOmegaMainMenu = {
    attach: function (context, settings) {

      //adds classes to main menu elements by level
      //to standardlize classes across superfish and menu-block modules
      $('ul#superfish-1 > li').addClass('main-menu-top-li');
      $('.main-menu-top-li > ul').addClass('main-menu-second-ul');
      $('#block-menu-block-custom-1 ul.menu').addClass('main-menu-second-ul');
      $('.main-menu-second-ul > li').addClass('main-menu-second-li');
      $('.main-menu-second-ul > li > ul > li').addClass('main-menu-third-plus-li').removeClass('main-menu-second-li');
    }
  };

  /*****************************************************
  *** Adds/alters classes for mean-menu
  *****************************************************/
  Drupal.behaviors.wcmOmegaMeanMenu = {
    attach: function (context, settings) {
      $(".mean-container .mean-nav ul li span").addClass("nolink");
    }
  };


  /*****************************************************
  *** adds functionality for search box in the menu bar
  *****************************************************/
  Drupal.behaviors.wcmOmegaSearchBlock = {
    attach: function (context, settings) {

      var toggleButton      = $('#search-block-toggle');
      var searchBox         = $('.l-region--main-menu #search-block-form');

      /**
       * The following reacts to clicks anywhere. If the click is within the
       * search box, the search box will stay visible. If the click is outside the
       * search box, the search box will be dismissed.
       */
      $(document).click(function(event) {
        toggleSearchBox(event);
      });

      toggleButton.keypress(function(event){
        if (event.which === 13 || event.which === 32) {
          event.preventDefault();
          toggleSearchBox(event);
        }
      });

      /* Hide search block form when it loses focus
       * We can't use jQuery's focusout() method because IE 11 is stupid. So, so stupid.
       */
      var searchForm = document.getElementById('block-search-form');

      if (searchForm) {
        searchForm.addEventListener("focusout", function(e) {
          if ($(e.relatedTarget).closest($(this)).length == 0) {
            hideHoverSearchBox_do();
          }
        }, false);
      }

      function toggleSearchBox(event) {
        var clickTarget = $(event.target).parent('#search-block-toggle').andSelf().eq(0);

        if (clickTarget.attr('id') == toggleButton.attr('id')) {
          // Clicked the toggle button.
          if (searchBox.is(':visible') ){
            // Turn sticky visibility on.
            hideHoverSearchBox();
          }
          else {
            showHoverSearchBox();
          }
        }
        else if (clickedSearchBox(event)) {
          return;
        }
        else if (searchBox.is(':visible')) {
          // This only matters if the search box is currently visible.
          hideHoverSearchBox(true);
        }
      }

      /*
       * Determines if the target clicked was inside the search box.
       */
      function clickedSearchBox(event) {
        var clickTarget = $(event.target).eq(0);
        var inSearchBox = false;
        // Check if the search box itself was clicked.
        if (searchBox.get(0) == clickTarget.get(0)) {
          // Search box was clicked.
          inSearchBox = true;
        }
        else {
          /*
           * Compare the click target against all the elements in the search box
           * to determine if the clicked element is part of it.
           */
          var elements = searchBox.find('*');
          elements.each(function(index) {
            if (elements.eq(index).get(0) == clickTarget.get(0)) {
              // Search box element was clicked.
              inSearchBox = true;
              return;
            }
          });
        }
        return inSearchBox;
      }

      /*
       * Displays the hover search box.
       */
      function showHoverSearchBox() {
        searchBox.stop(true, true).clearQueue().fadeIn('fast');
        toggleButton.attr('aria-expanded', 'true').find('.search-toggle-icon').addClass('fa-search-minus');
      }

      /*
       * Hides the hover search box when applicable.
       *
       * @param force
       *   If set to TRUE, forces the search box to be hidden and disables
       *   the sticky visibility parameter.
       */
      function hideHoverSearchBox(force) {
        if (force != undefined) {
          if (force === true) {
            // Hide the box and turn sticky visibility off.
            hideHoverSearchBox_do();
            return;
          }
        }

        if (!searchBox.is(':focus')) {
          hideHoverSearchBox_do();
        }
      }

      function hideHoverSearchBox_do() {
        // Handles the actual hiding, without any logical operations.
        searchBox.stop(true, true).fadeOut('fast');
        toggleButton.attr('aria-expanded', 'false').find('.search-toggle-icon').removeClass('fa-search-minus');
      }
    }
  };
})(jQuery);
